"""

Revision ID: 048bac60974f
Revises: 14b821135552
Create Date: 2025-11-13 15:31:56.901014

"""

from typing import Sequence, Union

from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import sqlite

# revision identifiers, used by Alembic.
revision: str = "048bac60974f"
down_revision: Union[str, None] = "14b821135552"
branch_labels: Union[str, Sequence[str], None] = None
depends_on: Union[str, Sequence[str], None] = None


def upgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###

    with op.batch_alter_table("portfolio", schema=None) as batch_op:
        batch_op.add_column(sa.Column("amount", sa.Float(), nullable=True))
        batch_op.drop_column("total_value")

    # ### end Alembic commands ###


def downgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    with op.batch_alter_table("portfolio", schema=None) as batch_op:
        batch_op.add_column(sa.Column("total_value", sa.FLOAT(), nullable=True))
        batch_op.drop_column("amount")

    # ### end Alembic commands ###



"""

Revision ID: c06dc04e782b
Revises: 048bac60974f
Create Date: 2025-11-14 10:01:56.688468

"""

from typing import Sequence, Union

from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import sqlite

# revision identifiers, used by Alembic.
revision: str = "c06dc04e782b"
down_revision: Union[str, None] = "048bac60974f"
branch_labels: Union[str, Sequence[str], None] = None
depends_on: Union[str, Sequence[str], None] = None


def upgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    with op.batch_alter_table("analysis", schema=None) as batch_op:
        batch_op.add_column(sa.Column("price_uptrend", sa.Date(), nullable=True))
        batch_op.add_column(sa.Column("sma_uptrend", sa.Date(), nullable=True))
        batch_op.add_column(sa.Column("rsi_uptrend", sa.Date(), nullable=True))
        batch_op.add_column(sa.Column("macd_12_26_9_uptrend", sa.Date(), nullable=True))
        batch_op.create_index(
            "ix_analysis_macd_12_26_9_uptrend", ["macd_12_26_9_uptrend"], unique=False
        )
        batch_op.create_index(
            "ix_analysis_price_uptrend", ["price_uptrend"], unique=False
        )
        batch_op.create_index("ix_analysis_rsi_uptrend", ["rsi_uptrend"], unique=False)
        batch_op.create_index("ix_analysis_sma_uptrend", ["sma_uptrend"], unique=False)

    # ### end Alembic commands ###


def downgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    with op.batch_alter_table("analysis", schema=None) as batch_op:
        batch_op.drop_index("ix_analysis_sma_uptrend")
        batch_op.drop_index("ix_analysis_rsi_uptrend")
        batch_op.drop_index("ix_analysis_price_uptrend")
        batch_op.drop_index("ix_analysis_macd_12_26_9_uptrend")
        batch_op.drop_column("macd_12_26_9_uptrend")
        batch_op.drop_column("rsi_uptrend")
        batch_op.drop_column("sma_uptrend")
        batch_op.drop_column("price_uptrend")

    # ### end Alembic commands ###



"""

Revision ID: 14b821135552
Revises: 65662e214031
Create Date: 2025-11-13 12:33:13.970721

"""

from typing import Sequence, Union

from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import sqlite
import sqlmodel

# revision identifiers, used by Alembic.
revision: str = "14b821135552"
down_revision: Union[str, None] = "65662e214031"
branch_labels: Union[str, Sequence[str], None] = None
depends_on: Union[str, Sequence[str], None] = None


def upgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.create_table(
        "portfolio",
        sa.Column("name", sqlmodel.sql.sqltypes.AutoString(), nullable=False),
        sa.Column("current_assets", sa.JSON(), nullable=True),
        sa.Column("new_assets", sa.JSON(), nullable=True),
        sa.Column("total_value", sa.Float(), nullable=True),
        sa.PrimaryKeyConstraint("name"),
    )

    # ### end Alembic commands ###


def downgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###

    op.drop_table("portfolio")
    # ### end Alembic commands ###



from typing import List, Optional, Any

from pydantic import BaseModel, Field


class PortfolioNewAsset(BaseModel):
    symbol: str


class PortfolioAsset(PortfolioNewAsset):
    value: float


class Portfolio(BaseModel):
    name: str = "default"
    current_assets: List[PortfolioAsset] = Field(
        default_factory=lambda: [PortfolioAsset(symbol="", value=1000)]
    )
    new_assets: List[PortfolioNewAsset] = Field(default_factory=list)
    amount: Optional[float] = None

    def to_dict(self) -> dict[str, Any]:
        return self.model_dump(exclude={"name"})

    def default_name(self) -> bool:
        return self.name == "default"

    def valid(self) -> bool:
        return self.current_assets is not None or self.new_assets is not None
